package com.fsck.k9.mail.store.imap;


class Capabilities {
    public static final String IDLE = "IDLE";
    public static final String CONDSTORE = "CONDSTORE";
    public static final String SASL_IR = "SASL-IR";
    public static final String AUTH_XOAUTH2 = "AUTH=XOAUTH2";
    public static final String AUTH_OAUTHBEARER = "AUTH=OAUTHBEARER";
    public static final String AUTH_CRAM_MD5 = "AUTH=CRAM-MD5";
    public static final String AUTH_PLAIN = "AUTH=PLAIN";
    public static final String AUTH_EXTERNAL = "AUTH=EXTERNAL";
    public static final String LOGINDISABLED = "LOGINDISABLED";
    public static final String NAMESPACE = "NAMESPACE";
    public static final String COMPRESS_DEFLATE = "COMPRESS=DEFLATE";
    public static final String ID = "ID";
    public static final String STARTTLS = "STARTTLS";
    public static final String SPECIAL_USE = "SPECIAL-USE";
    public static final String UID_PLUS = "UIDPLUS";
    public static final String LIST_EXTENDED = "LIST-EXTENDED";
    public static final String MOVE = "MOVE";
}
