package app.k9mail.feature.account.common.ui

data class WizardNavigationBarState(
    val isNextEnabled: Boolean = true,
    val showNext: Boolean = true,
    val isBackEnabled: Boolean = true,
    val showBack: Boolean = true,
)
